<?php
class Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{
	protected function _initPlugin()
	{
		if(!Precurio_Session::sessionExists())$this->bootstrap('session');;
		$this->bootstrap('request');
		$this->bootstrap('controller');
		$this->bootstrap('routes');
		$this->bootstrap('db');
		$this->bootstrap('log');
		$this->bootstrap('config');
		$this->bootstrap('view');
		$frontController = Zend_Controller_Front::getInstance(); 		
		// Change to 'production' parameter under production environemtn
		$frontController->registerPlugin(new Initializer('development'));    
		
		// Dispatch the request using the front controller. 
		//$frontController->dispatch();
	}
	/**
     * Initialize view 
     * 
     * @return void
     */
	protected function _initView()
	{
		$this->bootstrap('controller');
		$this->bootstrap('log');
		$this->bootstrap('config');
		$this->bootstrap('languages');
		
		$frontController = Zend_Controller_Front::getInstance(); 
        $baseUrl = $frontController->getBaseUrl();
        
        $root = Zend_Registry::get('root');
        
        $themes = new Precurio_Themes();
        
        $theme = $themes->getUserTheme();
        $style = $themes->getUserStyle($theme);

        // Bootstrap layouts

        $layout = Zend_Layout::startMvc(array(
            'layoutPath' => $root .  '/application/default/layouts/'.$theme,
            'layout'=>'main_c'
        ));
		
        $config = Zend_Registry::get('config');
        
        $liveServer = $config->live;
        try
        {
            $user = UserUtil::getUser(Precurio_Session::getCurrentUserId());
            $pwd = Precurio_Session::getCurrentUserPwd();
        }
        //if this exception is not caught, it propogates to the ErrorController, in this 
        //situation, we dont want that.
        catch (Exception $e)//i.e there was a problem retrieving the session user
        {
            $user = null;
        }
        $username = $user==null ? '' : $user->getUsername();
        $password = $user==null ? '' : $pwd;

        $view = new Zend_View();
        $view->assign('theme',$theme);
        $view->assign('style',$style);
        $view->assign('themestyle',$theme.'/'.$style);
        $view->assign('company_name',$config->company_name);

        $view->headScript()->appendFile($baseUrl.'/library/js/tipsy/jquery.tipsy.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/easySlider1.5.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery.curvycorners.source.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery_ui_drag_n_drop.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery.listmenu-1.1.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jcarousellite.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery-easing-compatibility.1.2.pack.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery-easing.1.2.pack.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/coda-slider.1.1.1.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/always_at_bottom.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jquery.autofill.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/admin.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/facebox.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/ajaxfileupload.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/styleswitch.js')
        				->appendScript("var baseUrl='". $baseUrl ."';
        								var currentStyle = '".$style."';"); 
        $view->headScript()->appendFile($baseUrl.'/library/js/others/comment.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/chat.js')
                            ->appendScript("var service='". $liveServer->service ."';
                                            var domain='". $liveServer->domain ."';
                                            var username='".$this->encrypt($username)."';
                                            var password='".$this->encrypt($password)."';");
        $view->headScript()->appendFile($baseUrl.'/library/js/strophe/strophe.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/strophe/sha1.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/strophe/md5.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/strophe/b64.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/task.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/layout.js');
        $view->headScript()->appendFile($baseUrl.'/library/js/others/jqprint.js');
        $view->headLink()->setStylesheet($baseUrl."/library/css/$theme/theme.css");
        $styles = $themes->getThemeStyles($theme);
        foreach($styles as $aStyle)
        {
            $view->headLink(array('title'=>$aStyle,'rel'=>($style == $aStyle ? 'stylesheet' : 'alternate stylesheet'),'href'=>$baseUrl."/library/css/$theme/$aStyle/style.css",'type'=>'text/css'))    ;
        }    

        $view->headLink()->appendStylesheet($baseUrl."/library/css/tipsy/tipsy.css");
        $view->headLink()->appendStylesheet($baseUrl."/library/css/facebox/facebox.css");
        $view->addHelperPath($root .'/application/default/views/helpers', 'Default_View_Helper');
        $view->addHelperPath($root .'/application/workflow/views/helpers', 'Workflow_View_Helper');
        $view->addHelperPath($root .'/application/news/views/helpers', 'News_View_Helper');
        $view->addHelperPath($root .'/application/task/views/helpers', 'Task_View_Helper');
        $view->addHelperPath($root .'/application/cms/views/helpers', 'Cms_View_Helper');
        $view->addHelperPath($root .'/application/chat/views/helpers', 'Chat_View_Helper');
        $view->addHelperPath($root .'/application/user/views/helpers', 'User_View_Helper');
        $view->addHelperPath($root .'/application/employee/views/helpers', 'Employee_View_Helper');
        
        
        Zend_View_Helper_PaginationControl::setDefaultViewPartial('page_control.phtml');
        
         $config = new Zend_Config_Xml($root.'/application/configs/navigation.xml');
 
            $navigation = new Zend_Navigation($config);
         $view->navigation($navigation);
         
         $viewRenderer = new Zend_Controller_Action_Helper_ViewRenderer();
        $viewRenderer->setView($view);
        Zend_Controller_Action_HelperBroker::addHelper($viewRenderer);
        
	}
	/**
     * Initialize Controller paths 
     * 
     * @return void
     */
	protected function _initController()
	{
		$front = Zend_Controller_Front::getInstance();
		$root = Zend_Registry::get('root');
		
		$front->addControllerDirectory($root . '/application/default/controllers');
        $front->addControllerDirectory($root.'/application/workflow/controllers', 'workflow');
        $front->addControllerDirectory($root.'/application/user/controllers', 'user');
        $front->addControllerDirectory($root.'/application/chat/controllers', 'chat');
        $front->addControllerDirectory($root.'/application/task/controllers', 'task');
        $front->addControllerDirectory($root.'/application/news/controllers', 'news');
        $front->addControllerDirectory($root.'/application/employee/controllers', 'employee');
        $front->addControllerDirectory($root.'/application/event/controllers', 'event');
        $front->addControllerDirectory($root.'/application/contact/controllers', 'contact');
        $front->addControllerDirectory($root.'/application/helpdesk/controllers', 'helpdesk');
        $front->addControllerDirectory($root.'/application/poll/controllers', 'poll');
        $front->addControllerDirectory($root.'/application/cms/controllers', 'cms');
        $front->addControllerDirectory($root.'/application/search/controllers', 'search');
        $front->addControllerDirectory($root.'/application/document/controllers', 'document');
		$front->addControllerDirectory($root.'/application/admin/controllers', 'admin');
		$front->addControllerDirectory($root.'/application/report/controllers', 'report');
		$front->addControllerDirectory($root.'/application/install/controllers', 'install');
		
		return $front;
	}
	protected function _initConfig()
	{
		$this->bootstrap('controller');
		
		$root = Zend_Registry::get('root');
		$config = new Zend_Config_Ini($root.'/application/configs/precurio.ini',null,array('allowModifications'=>true));
		Zend_Registry::set('config',$config);
    	return $config;
	}
	/**
     * Initialize data bases
     * 
     * @return void
     */	
	protected function _initDb()
    {
    	$this->bootstrap('controller');
    	$this->bootstrap('config');
    	
    	$root = Zend_Registry::get('root');
    	$config = Zend_Registry::get('config');
    	
    	$active_db = $config->active_database;
    	$db_config = new Zend_Config_Ini($root.'/application/configs/precurio.ini',$active_db);
    	
    	$db_config = $db_config->toArray();
    	$db_config = $db_config['database'];
    	
    	$adapter = $db_config['adapter'];
    	$params = array();
		foreach ($db_config as $key=>$value) {
			if($key == 'adapter')continue;
			$params[$key] = $value;
		}

		$db = Zend_Db::factory($adapter,$params);
		if($config->database_auto_connect)
		{
			try {
				$db->getConnection();
			}
			catch(Zend_Db_Exception $e)
			{
				throw new Precurio_Exception($e->getMessage(),Precurio_Exception::EXCEPTION_DATABASE_CONNECTION);
			}
			catch(Zend_Exception $e)
			{
				throw new Precurio_Exception($e->getMessage(),Precurio_Exception::EXCEPTION_DATABASE_CONNECTION);
			}
			$db->closeConnection();
		}

		Zend_Registry::set('db',$db);
		Zend_Db_Table::setDefaultAdapter($db);
		
		$frontendOptions = array('automatic_serialization' => true);//cache is valid for ever
		$backendOptions = array('cache_dir' => $root.'/application/tmp/');
		
		$cache = Zend_Cache::factory('Core','File',$frontendOptions,$backendOptions);
		
    	Zend_Db_Table_Abstract::setDefaultMetadataCache($cache);
    	return $db;
    }
    /**
     * Initialize action helpers
     * 
     * @return void
     */
    protected function _initHelpers()
    {
    	$this->bootstrap('controller');
    	// register the default action helpers
    	$root = Zend_Registry::get('root');
    	Zend_Controller_Action_HelperBroker::addPath($root .'/application/default/helpers', 'Precurio_Helper');
    	
    }
 	protected function _initLanguages()
    {
    	$this->bootstrap('controller');
    	$this->bootstrap('config');
    	
    	$root = Zend_Registry::get('root');

    	$frontendOptions = array('automatic_serialization' => true,'lifetime' => 7200);//cache is valid for 2 hrs
		$backendOptions = array('cache_dir' => $root.'/application/tmp/');
		
		$cache = Zend_Cache::factory('Core',
		'File',
		$frontendOptions,
		$backendOptions);
		
		Zend_Translate::setCache($cache);
		
		//get locale settings for registry
    	$registry = Zend_Registry::getInstance();
		$config = $registry->get('config');
		
		try 
    	{
			$locale = new Zend_Locale($config->user_locale);
			if(!$locale->isLocale($locale))throw new Zend_Locale_Exception("User locale is invalid");
		} 
		catch (Zend_Locale_Exception $e) //some sort of error with the user locale
		{
			$config->user_locale = $config->default_locale;//replace the user_locale with a valid application locale
			$locale = new Zend_Locale($config->default_locale);
		}
		
		$lang = $locale->getLanguage();// change to $config->user_locale at production
		$translate = new Zend_Translate('gettext', $root .'/application/languages/'.$lang.'/LC_MESSAGES/precurio.mo',$lang);
		
		$translate->setLocale($locale);

		
		$registry->set('Zend_Translate', $translate);
		$registry->set('Zend_Locale', $locale);
	
    }
  	/**
     * Initialize routes
     * 
     * @return void
     */
    protected function _initRoutes()
    {
		$this->bootstrap('controller');
		
    	$router = Zend_Controller_Front::getInstance()->getRouter();
																			
    	$router->addRoute('workflow',new Zend_Controller_Router_Route('workflow/new/:id',
																			array('module'=>'workflow',
																			'controller' => 'process',
																			'action' => 'new',
																			'id' =>1)));
																			
		$router->addRoute('workflow2',new Zend_Controller_Router_Route('workflow/view/:id',
																			array('module'=>'workflow',
																			'controller' => 'process',
																			'action' => 'view',
																			'id' =>1)));

		$router->addRoute('profile',new Zend_Controller_Router_Route('user/profile/view/:id/:page/:param1/:param2/:cpage',
																			array('module'=>'user',
																			'controller' => 'profile',
																			'action' => 'view',
																			'id'=>0,
																			'page'=>'recent',
																			'param1'=>'0',
																			'param2'=>'0',
																			'cpage'=>'0')));																	
	    															
																
    }
	/**
     * Initialize Session
     * 
     * @return void
     */
    protected function _initSession()
    {
    	$root = Zend_Registry::get('root');
    	Zend_Session::start(array('save_path'=>$root.'/application/tmp/',
        	'name'=>'PRECURIOSESSID'
        ));
    }
	/**
     * Initialize logs
     * 
     * @return void
     */
    protected function _initLog()
    {
    	$log = new Precurio_Log();
    	Zend_Registry::set('log',$log);
    	return $log;
    	
    }
	protected function _initRequest()
	{
	    $front = Zend_Controller_Front::getInstance();
	    $request = $front->getRequest();
	    if (null === $request) {
	        $request = new Zend_Controller_Request_Http();
	        $front->setRequest($request);
	    }
	    return $request;
	}
	private function encrypt($str)
	{
		$key = rand(1,6);
		$arr = str_split($str);
		for($i=0;$i<count($arr);$i++)
		{
			$arr[$i] = chr(ord($arr[$i]) +$key);
		}
		$arr[] = chr($key+65);
		return implode('',$arr);
		
	}
}
?>