<?php

require_once ('Zend/Session.php');
require_once ('user/models/vo/User.php');
class Precurio_Session extends Zend_Session {
	const NAMESPACE_USER = "user";
	const NAMESPACE_WORKFLOW = "workflow";
	const NAMESPACE_CHAT = 'chat';
	const NAMESPACE_CONTENT = 'content';
	const NAMESPACE_REPORT = 'report';
	const NAMESPACE_APP = 'app';
	
	public static function setCurrentUser($user)
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_USER);
		$ns->currentUser = $user;
		self::rememberMe();		
		$root = Zend_Registry::get('root');
        $frontendOptions = array('automatic_serialization' => false);//cache is valid for ever
		$backendOptions = array('cache_dir' => $root.'/application/tmp/');
		
		$cache = Zend_Cache::factory('Output','File',$frontendOptions,$backendOptions);
		$cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,
   		 array('user'));
	}
	public static function logOut()
	{
	
		self::forgetMe();
		Zend_Session::expireSessionCookie();
		Zend_Session::destroy();
	}
	public static function getCurrentUserId()
	{
		try
		{
			$ns = new Zend_Session_Namespace(self::NAMESPACE_USER);
			if(!isset($ns->currentUser))//throw a re-login error because session has expired
			{
				$request = Zend_Controller_Front::getInstance()->getRequest();
				if(!($request->getControllerName() == 'login' || $request->getActionName() == 'remind' || $request->getControllerName() == 'error' || $request->getModuleName() == 'default' || $request->getModuleName() == NULL || $request->getModuleName() == 'chat'))
				{
					if($request->isDispatched())
					{
						$ns->lastRequest = serialize($request->getRequestUri());
					}
				}
				throw new Precurio_Exception(PrecurioStrings::SESSIONEXPIRED,Precurio_Exception::EXCEPTION_SESSION_EXPIRED);
			}
			
			$user = $ns->currentUser;
			
			return $user->id;
		}
		catch (Zend_Session_Exception $e)//i.e there was a problem retrieving the session user
		{
			return 0;
		}
	}
	public static function getCurrentUserPwd()
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_USER);
		if(!isset($ns->currentUser))//throw a re-login error because session has expired
			throw new Precurio_Exception(PrecurioStrings::SESSIONEXPIRED,Precurio_Exception::EXCEPTION_SESSION_EXPIRED);
		
		$user = $ns->currentUser;
		return $user->credential;
	}
	/**
	 * @return Precurio_License
	 */
	public static function getLicense()
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_APP);
		if(!isset($ns->license))
		{
			return  new Precurio_License();
		}
			
		return $ns->license;
	}
	/**
	 * @param $license Precurio_License
	 * @return null
	 */
	public static function setLicense($license)
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_APP);
		$ns->license = $license;
		$ns->setExpirationSeconds(3600);
	}
	public static function setStyle($style)
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_USER);
		$ns->style = $style;
	}
	public static function getStyle()
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_USER);
		if(!isset($ns->style))
		{
			return null;
		}
			
		return $ns->style;
	}
	/**
	 * @return Precurio_ChatSession
	 */
	public static function getChatSession()
	{
		$ns = new Precurio_ChatSession(self::NAMESPACE_CHAT);
		return $ns;
	}
	/**
	 * @param Zend_Form $form
	 * @return void
	 */
	public static function setCurrentForm($form)
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_WORKFLOW);
		$ns->currentForm = serialize($form);
		//expire current form after 60 mins i.e a user has only an hr to complete a form.
		$ns->setExpirationSeconds(6000,'currentForm');
	}
	/**
	 * @return Zend_Form
	 */
	public static function getCurrentForm()
	{
		$ns = new Zend_Session_Namespace(self::NAMESPACE_WORKFLOW);
		if(!isset($ns->currentForm)) 
		throw new Precurio_Exception(PrecurioStrings::FORM_SESSION_EXPIRED,Precurio_Exception::EXCEPTION_FORM_SESSION_EXPIRED);
			
		return unserialize($ns->currentForm);
	}
}

?>