<?php

require_once ('Precurio/FileFn.php');

class Precurio_Upgrade extends Precurio_FileFn {
	
	private $_path;
	private $_log;
	private $_appPath;//path to precurio
	public function __construct()
	{
		$this->_log = Zend_Registry::get('log');
		$this->_appPath = Zend_Registry::get('root');
	}
	/**
	 * @param $zipPath - full file name (i.e including path) of the upgrade zip file
	 * @return void
	 */
	public function upgrade($zipPath)
	{
		$zip = new dUnzip2($zipPath);
		$basefilename = preg_replace("/(.*)\.([^.]+)$/","\\1", $zipPath);
		$zip->unzipAll($basefilename);
		
		$filename = basename($zipPath,'.zip');
		$upgradePath = $basefilename.'/'.$filename;//$upgradePath is path to file contents of unziped upgrade file
		
		$this->_path = $upgradePath;
		$cmds = file_get_contents($upgradePath.'/upgrade.bat');
		$cmds = explode(";",$cmds);
		foreach($cmds as $cmd)
			$this->executeCommand($cmd);
		
		$this->deldir($basefilename);
		$zip->close();
		unset($zip);
		unlink($zipPath);
		echo "Upgrade Successful!";
	}
	
	private function executeCommand($cmd)
	{
		$cmd = trim($cmd);
		$cmd = explode(',',$cmd);
		switch(strtoupper($cmd[0]))
		{
			case 'COPY':
				$this->filecopy($cmd[1],$cmd[2]);
				break;
				
			case 'DELETE':
				$this->filedelete($cmd[1]);
				break;
			case 'RENAME':
				$this->filerename($cmd[1],$cmd[2],$cmd[3]);
				break;
			case 'SET':
				if(Precurio_Utils::isNull($cmd[3]))$cmd[3] = "";
				$this->configset($cmd[1],$cmd[2],$cmd[3]);
				break;
			case 'EXECUTE':
				$this->executesql($cmd[1]);
				break;
			default :
				echo "Unknown Command";
				exit();
				
		}
	}
	/**
	 * @param $dir string directory path,MUST contain a leading slash
	 * @return unknown_type
	 */
	private function realDir($dir)
	{
		return $this->_appPath.$dir;
	}
	
	/**
	 * Handle copying of file or folders
	 * @param $from
	 * @param $to
	 * @return void
	 */
	private function filecopy($from, $to)
	{
		$from = $this->_path.$from; $to = $this->realDir($to);
		$this->copy($from,$to);
	}
	/**
	 * Handles file or folder delete
	 * @param $file
	 * @return void
	 */
	private function filedelete($file)
	{
		$file = $this->realDir($file);
		if(is_dir($file))
		{
			$this->deldir($file);
		}
		else
		{
			if(file_exists($file))
				unlink($file);
		}
	}
	/**
	 * @param $dir path to folder containing file/folders to rename
	 * @param $old
	 * @param $new
	 * @return void
	 */
	private function filerename($dir,$old,$new)
	{
		$dir = $this->realDir($dir);
		$oldWD = getcwd();
		chdir($path);
		rename($old, $new);
		chdir($oldWD);
	}
	private function configset($param,$value,$section="")
	{
		$root = Zend_Registry::get('root');
		$config = new Zend_Config_Ini($root.'/application/configs/precurio.ini',null,array('allowModifications'=>true));
		
		$config = $config->toArray();
		
		$tempConfig = $config;
		if(!Precurio_Utils::isNull($section))
		{
			 $tempConfig = $config[$section]; //remove section from config
		}
		
		$paramArr = explode(".",$param);
		$this->setParam($tempConfig,$paramArr,$value);
		
		if(!Precurio_Utils::isNull($section))
		{
			 $config[$section] = $tempConfig; //attach section back to overall config
		}
		else
			$config = $tempConfig;
		
		$newConfig = new Zend_Config($config,true);
		
		$item = new Precurio_ConfigWriter(array('config'   => $newConfig,
                                           'filename' => $this->_appPath.'/application/configs/precurio.ini'));
		
		$item->write();
	}
	
	private function executesql($sqlFile)
	{
		$sqlFile = $this->_path.'/'.$sqlFile;
		$sql = file_get_contents($sqlFile);
		$db = Zend_Registry::get('db');
		$db->query($sql);
	}
	private function setParam(&$arr,$params,$value)
	{
		if(count($params) > 1)
		{
			$param = array_shift($params);
			if(!isset($arr[$param]))
				$arr[$param] = array();
			return $this->setParam($arr[$param],$params,$value);
		}
		else
		{
			$arr[$params[0]] = $value;
		}
		return $arr;
	}
	

}

?>